<?php
namespace PME\Controllers\Medicao;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;

class RetiradaInventarioController extends \PME\Services\TeknisaCrud {
  protected $crudService;
	protected $parameterBag;
	protected $environment;
	protected $dataSourceName;
	protected $connection;
  protected $lista;
  protected $newCode;
  protected $numLista;

  public function __construct($crudService, $parameterBag, $connection, $environment, $lista, $newCode) {
    parent::__construct($crudService, $parameterBag, $environment);
    $this->connection  = $connection;
    $this->environment = $environment;
    $this->lista       = $lista;
    $this->newCode     = $newCode;
	}
  
  public function configRetirada(DTO\Request $request, DTO\Response $response) {
    try {
      $params = $request->getParameters();
      $query = "SELECT IDPRINVENRETI, CDSERVRETPME, CDTPRETIPADR
                  FROM PARAMPLANEJA
                WHERE CDFILIAL = :P_CDFILIAL";
      $params = array (
        "P_CDFILIAL" => $params['CDCENTRAL']
      );
      $data = $this->connection->fetchAll($query, $params);

      $response->addDataSet(new DataSet("result", $data));
    } catch (\Exception $e) {
      $response->setError(new Error($e->getMessage(), 500));
    }
  }
  
  public function returnRowsCliente(&$rows){
    try{
      $query = "SELECT SERVCLIEPME.CDFILIAL, SERVCLIEPME.CDSERVFILI, SERVCLIEPME.CDFILCENTPME, SERVCLIEPME.CDTIPOCARD, TO_CHAR(EFETCLIEPME.DTEFETCLPME, 'DD/MM/YYYY') AS DTEFETCLPME, TO_CHAR(EFETCLIEPME.DTEFETCLPME, 'DD/MM/YYYY') AS DTCARDIAPME, SERVCLIEPME.CDCLIENTE, CLIENTE.NMRAZSOCCLIE, EFETCLIEPME.QTCOMEFCLPME, EFETCLIEPME.QTFUNEFCLPME, EFETCLIEPME.QTDOAEFCLPME, SERVCLIEPME.NRORG FROM CLIENTE, SERVCLIEPME LEFT JOIN EFETCLIEPME ON EFETCLIEPME.DTEFETCLPME = :DTEFETCLIE AND SERVCLIEPME.CDFILCENTPME = EFETCLIEPME.CDFILCENTPME AND SERVCLIEPME.CDTIPOCARD = EFETCLIEPME.CDTIPOCARD AND SERVCLIEPME.CDFILIAL = EFETCLIEPME.CDFILIAL AND SERVCLIEPME.CDSERVFILI = EFETCLIEPME.CDSERVFILI AND SERVCLIEPME.CDCLIENTE = EFETCLIEPME.CDCLIENTE WHERE SERVCLIEPME.CDFILCENTPME = :CDFILCENTPME AND SERVCLIEPME.CDTIPOCARD = :CDTIPOCARD AND SERVCLIEPME.CDFILIAL = :CDFILIAL AND SERVCLIEPME.CDSERVFILI = :CDSERVFILI AND SERVCLIEPME.CDCLIENTE = CLIENTE.CDCLIENTE AND SERVCLIEPME.NRORG = :NRORG ORDER BY CLIENTE.NMRAZSOCCLIE";
      $params = array (
        "DTEFETCLIE" => $rows[0]['DTEFETCLIE'],
        "CDFILCENTPME" => $rows[0]['CDFILCENTPME'],
        "CDTIPOCARD" => $rows[0]['CDTIPOCARD'],
        "CDFILIAL" => $rows[0]['CDFILIAL'],
        "CDSERVFILI" => $rows[0]['CDSERVFILI'],
        "NRORG" => $rows[0]['NRORG']
      );
      $data = $this->connection->fetchAll($query, $params);
      foreach ($data as $row){
        $valid = true;
        foreach ($rows as $editable) {
          if ($editable['CDCLIENTE'] == $row['CDCLIENTE']) {
            $valid = false;  
          }  
        }
        if ($valid) {
          array_push($rows, $row);
        }
      }
    } catch (\Exception $e) {
	    throw new $e;
	  }
  }

  public function saveItemsRomaneio(DTO\Request $request, DTO\Response $response) {
    try{
      $rows = $request->getRow();
      $nrSeq = $this->getMaxSeqItem($rows[0]['CDFILIAL'], $rows[0]['NRINVENTRETI']) + 1;
      
      $this->connection->beginTransaction();

      $queryInsert = "INSERT INTO ITEMINVENTRETI (CDFILIAL, NRINVENTRETI, NRITINVENTRETI, QTITINVENTRETI, CDPRODUTO, IDORIGLANCPRINV)
                      VALUES (:CDFILIAL, :NRINVENTRETI, :NRITINVENTRETI, :QTITINVENTRETI, :CDPRODUTO, :IDORIGLANCPRINV)";

      $queryUpdate = "UPDATE ITEMINVENTRETI SET QTITINVENTRETI = :QTITINVENTRETI
                      WHERE CDFILIAL = :CDFILIAL AND NRINVENTRETI = :NRINVENTRETI AND NRITINVENTRETI = :NRITINVENTRETI 
                      AND CDPRODUTO = :CDPRODUTO AND IDORIGLANCPRINV = :IDORIGLANCPRINV";

      foreach($rows as $row) {
        if($row['__is_new']){
          $row['IDORIGLANCPRINV'] = 0;
          $row['NRITINVENTRETI'] = $nrSeq;
          $nrSeq += 1;
          $stmt = $this->connection->prepare($queryInsert);
        } else {
          $stmt = $this->connection->prepare($queryUpdate);
        } 

        $params = array(
          'CDFILIAL' => $row['CDFILIAL'],
          'NRINVENTRETI' => $row['NRINVENTRETI'],
          'NRITINVENTRETI' => $row['NRITINVENTRETI'],
          'QTITINVENTRETI' => $row['QTITINVENTRETI'],
          'CDPRODUTO' => $row['CDPRODUTO'],
          'IDORIGLANCPRINV' => $row['IDORIGLANCPRINV']
        );

        if($row['__is_new']) $stmt = $this->connection->prepare($queryInsert);
        else $stmt = $this->connection->prepare($queryUpdate);
        
        $stmt->execute($params);
      }
      $this->connection->commit();
      $response->addDataSet(new DataSet("success", array(true)));
    } catch (\Exception $e) {
      $this->connection->rollback();
      $response->setError(new Error($e->getMessage(), 500));
    }
  }

  public function deleteItemRomaneio(DTO\Request $request, DTO\Response $response){
    try{
      $rows = $request->getRow();
      $this->connection->beginTransaction();

      $this->deleteRomaneioEItem($rows[0]);

      $this->connection->commit();
      $response->addDataSet(new DataSet('response', ['true']));
    } catch (\Exception $e) {
      $this->connection->rollback();
      $response->setError(new Error($e->getMessage(), 500));
    }
  }

  public function deleteRomaneio(DTO\Request $request, DTO\Response $response){
    try{
      $rows = $request->getRow();
      $this->connection->beginTransaction();

      foreach($rows as $row){
        $this->deleteRomaneioEItem($row);
      }

      $this->connection->commit();
      $response->addDataSet(new DataSet('response', ['true']));
    } catch (\Exception $e) {
      $this->connection->rollback();
      $response->setError(new Error($e->getMessage(), 500));
    }
  }

  private function deleteRomaneioEItem($row){
    $queryItems = "DELETE FROM ITEMINVENTRETI WHERE CDFILIAL = :CDFILIAL AND NRINVENTRETI = :NRINVENTRETI";
    $queryRomaneio = "DELETE FROM INVENTARIORETI WHERE CDFILIAL = :CDFILIAL AND NRINVENTRETI = :NRINVENTRETI";
    $params = array(
      "CDFILIAL" => $row["CDFILIAL"],
      "NRINVENTRETI" => $row["NRINVENTRETI"]
    );

    $this->connection->prepare($queryItems)->execute($params);
    $this->connection->prepare($queryRomaneio)->execute($params);
  }

  private function getMaxSeqItem($cdfilial, $nrinvent) {
    $query = "SELECT NVL(MAX(NRITINVENTRETI), 0) MAXITINVENT FROM ITEMINVENTRETI WHERE CDFILIAL = :P_CDFILIAL AND NRINVENTRETI = :P_NRINVENTRETI";
		$params = array(
			"P_CDFILIAL" => $cdfilial,
			"P_NRINVENTRETI" 	=> $nrinvent
		);
		$stmt = $this->connection->prepare($query);
    $stmt->execute($params);
    $data = $stmt->fetchAll();
    return $data[0]['MAXITINVENT'];
  }

  private function getMaxSeqRomaneio($cdfilial) {
    $query = "SELECT TO_CHAR(NVL((MAX(NRINVENTRETI) + 1) , 1)) AS NRINVENTRETI FROM INVENTARIORETI WHERE CDFILIAL = :P_CDFILIAL";
		$params = array(
			"P_CDFILIAL" => $cdfilial
		);
		$stmt = $this->connection->prepare($query);
    $stmt->execute($params);
    $data = $stmt->fetchAll();
    return $data[0]['NRINVENTRETI'];
  }

  public function saveRomaneio(DTO\Request $request, DTO\Response $response){
    try{
      $row = $request->getParameters();

      $romaneio = $row['romaneio'];
      $item = $row['item'];

      $this->connection->beginTransaction();
      
      //insert Romaneio
      if(empty($item['NRINVENTRETI'])){
        $nrSeqRom = $this->getMaxSeqRomaneio($romaneio['CDFILIAL']);
        $queryRomaneio = "INSERT INTO INVENTARIORETI (CDFILIAL, NRINVENTRETI, DTINVENTRETI, DSINVENTRETI, IDRETIEFET)
                          VALUES (:CDFILIAL, :NRINVENTRETI, :DTINVENTRETI, :DSINVENTRETI, :IDRETIEFET)";
        $params = array(
          'CDFILIAL' => $romaneio['CDFILIAL'],
          'NRINVENTRETI' => $nrSeqRom,
          'DTINVENTRETI' => $romaneio['DTINVENTRETI'],
          'DSINVENTRETI' => $romaneio['DSINVENTRETI'],
          'IDRETIEFET'   => 'N'
        );
        $stmt = $this->connection->prepare($queryRomaneio);   
        $stmt->execute($params);
        $item['NRINVENTRETI'] = $nrSeqRom;
        
        $nrSeqItem = 1;
      }else $nrSeqItem = $this->getMaxSeqItem($item['CDFILIAL'], $item['NRINVENTRETI']) + 1;
                         
      //insert Items
      $queryItem = "INSERT INTO ITEMINVENTRETI (CDFILIAL, NRINVENTRETI, NRITINVENTRETI, QTITINVENTRETI, CDPRODUTO, IDORIGLANCPRINV)
                    VALUES (:CDFILIAL, :NRINVENTRETI, :NRITINVENTRETI, :QTITINVENTRETI, :CDPRODUTO, :IDORIGLANCPRINV)";

      $params = array(
        'CDFILIAL' => $romaneio['CDFILIAL'],
        'NRINVENTRETI' => $item['NRINVENTRETI'],
        'NRITINVENTRETI' => $nrSeqItem,
        'QTITINVENTRETI' => $item['QTITINVENTRETI'],
        'CDPRODUTO' => $item['CDPRODUTO'],
        'IDORIGLANCPRINV' => 0
      );

      $stmt = $this->connection->prepare($queryItem);   
      $stmt->execute($params);
      $nrSeqItem += 1;
      

      $this->connection->commit();
      $response->addDataSet(new DataSet('nrseqinvent', [$item['NRINVENTRETI']]));
    } catch (\Exception $e) {
      $this->connection->rollback();
      $response->setError(new Error($e->getMessage(), 500));
    }
  }
}